# -*- coding: utf-8 -*-
"""
Created on Fri Mar 18 11:40:14 2022

@author: michael

!python -m PyQt5.uic.pyuic -x gui.ui -o gui.py
!python -m PyQt5.uic.pyuic -x dialog.ui -o dialog.py
!pyinstaller main.py -F --distpath . --debug=all -n"NSNA" -i"iconEdit.ico"
"""

from PyQt5.QtWidgets import QApplication, QWidget, QMainWindow, QFileDialog
from PyQt5.QtCore import QObject, QThread, pyqtSignal, Qt
from PyQt5.Qt import QDialog, QApplication
from PyQt5 import QtWidgets
from PyQt5 import QtGui
from gui import Ui_MainWindow
from dialog import Ui_Dialog
import sys
import ee, pandas as pd
import methods
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure

class WorkerOccurrences(QObject):
    finished = pyqtSignal()
    
    def __init__(self, taxonName):
        super().__init__()
        self.taxonName = taxonName
        
    def run(self):
        try:
            self.ex = False
            self.occurrenceData = methods.getOccurrences(self.taxonName)
        except Exception as e:
            self.ex = True
            self.e = e
            
        self.finished.emit()
        
        
class WorkerSpectral(QObject):
    finished = pyqtSignal()
    
    def __init__(self, yearParams, filterParams):
        super().__init__()
        self.yearParams = yearParams
        self.filterParams = filterParams
        
    def run(self):
        try:
            self.ex = False
            self.spectralDict = methods.getSpectralValues(self.yearParams, self.filterParams)
        except Exception as e:
            self.ex = True
            self.e = e
            
        self.finished.emit()
        
        
class Dialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.dialog = Ui_Dialog()
        self.dialog.setupUi(self)
        

class MainWindow(Ui_MainWindow):
    def init(self, MainWindow):
        self.MainWindow = MainWindow
        self.MainWindow.setWindowIcon(QtGui.QIcon('iconEdit.ico'))
        self.plotShowed = False
        self.plot2Showed = False
        self.plot3Showed = False
        self.bands = ['BLUE', 'GREEN', 'RED', 'NIR', 'SWIR1', 'SWIR2']
        
    def interactions(self):
        self.taxonPushButton.clicked.connect(self.getOccurrences)
        self.filterPushButton.clicked.connect(self.getSpectralValues)
        self.bandComboBox.currentIndexChanged.connect(self.visualizeData)
        self.returnFilterPushButton.clicked.connect(lambda: self.stackedWidget.setCurrentIndex(1))
        self.exportDataPushButton.clicked.connect(self.exportData)
    
    def showDialog(self, e):
        self.show = Dialog(self.MainWindow)
        self.show.dialog.errorLabel.setText(str(e))
        self.show.setWindowFlags(self.show.windowFlags() & ~Qt.WindowContextHelpButtonHint)
        self.show.dialog.okPushButton.clicked.connect(lambda: self.show.close())
        app.alert(self.MainWindow, 0)
        self.show.exec()
        
    def getOccurrences(self):
        self.taxonName = self.taxonLineEdit.text()
        if self.taxonName != '':
            self.thread = QThread()
            self.worker = WorkerOccurrences(self.taxonName)
            self.worker.moveToThread(self.thread)
            self.thread.started.connect(self.workerOccurrencesStart) 
            self.thread.started.connect(self.worker.run)
            self.worker.finished.connect(self.thread.quit)
            self.worker.finished.connect(self.worker.deleteLater)
            self.thread.finished.connect(self.thread.deleteLater)
            self.thread.finished.connect(self.workerOccurrencesEnd)
            self.thread.start()
        else:
            self.showDialog('zadej latinský název druhu!')
            
    def workerOccurrencesStart(self):
        self.MainWindow.setCursor(Qt.BusyCursor)
        self.taxonPushButton.setEnabled(False)
        self.taxonLineEdit.setEnabled(False)        
        
    def workerOccurrencesEnd(self):
        self.MainWindow.unsetCursor()
        self.taxonPushButton.setEnabled(True)
        self.taxonLineEdit.setEnabled(True)
        self.occurrenceData = self.worker.occurrenceData
        
        if self.worker.ex:
            self.showDialog(str(self.worker.e))
        else:
            app.alert(self.MainWindow, 0)
            self.stackedWidget.setCurrentIndex(1)
        
    def getSpectralValues(self):
        filterParams = [self.occurrenceData,
                        self.filterSpinBox.value(),
                        self.filterSpinBox2.value(),
                        self.filterComboBox.currentIndex()
                        ]
        
        yearParams = [self.startYearSpinBox.value(),
                      self.endYearSpinBox.value(),
                      self.stepSpinBox.value()
                      ]
        
        if yearParams[0] > yearParams [1] or yearParams[2] > yearParams[1] - yearParams[0]:
            self.showDialog('chybně zadané hodnoty časového období!')
        else:
            if filterParams[3] in [4,6]:
                if filterParams[1] > filterParams[2]:
                    self.showDialog('druhá hodnota filtru přesnosti musí být větší než první!')
                else:
                    self.thread = QThread()
                    self.worker = WorkerSpectral(yearParams, filterParams)
                    self.worker.moveToThread(self.thread)
                    self.thread.started.connect(self.workerSpectralStart) 
                    self.thread.started.connect(self.worker.run)
                    self.worker.finished.connect(self.thread.quit)
                    self.worker.finished.connect(self.worker.deleteLater)
                    self.thread.finished.connect(self.thread.deleteLater)
                    self.thread.finished.connect(self.workerSpectralEnd)
                    self.thread.start()
                    
            else:
                self.thread = QThread()
                self.worker = WorkerSpectral(yearParams, filterParams)
                self.worker.moveToThread(self.thread)
                self.thread.started.connect(self.workerSpectralStart) 
                self.thread.started.connect(self.worker.run)
                self.worker.finished.connect(self.thread.quit)
                self.worker.finished.connect(self.worker.deleteLater)
                self.thread.finished.connect(self.thread.deleteLater)
                self.thread.finished.connect(self.workerSpectralEnd)
                self.thread.start()
                
                
    def workerSpectralStart(self):
        self.MainWindow.setCursor(Qt.BusyCursor)
        self.filterPushButton.setEnabled(False)
        self.startYearSpinBox.setEnabled(False)
        self.endYearSpinBox.setEnabled(False)
        self.stepSpinBox.setEnabled(False)
        self.filterComboBox.setEnabled(False)
        self.filterSpinBox.setEnabled(False)
        self.filterSpinBox2.setEnabled(False)
        
    def workerSpectralEnd(self):
        self.MainWindow.unsetCursor()
        self.spectralDict = self.worker.spectralDict
        self.filterPushButton.setEnabled(True)
        self.startYearSpinBox.setEnabled(True)
        self.endYearSpinBox.setEnabled(True)
        self.stepSpinBox.setEnabled(True)
        self.filterComboBox.setEnabled(True)
        self.filterSpinBox.setEnabled(True)
        self.filterSpinBox2.setEnabled(True)
        
        if self.worker.ex:
            self.showDialog(str(self.worker.e))
        else:
            app.alert(self.MainWindow, 0)
            self.visualizeData()
            self.visualizeData2()
            self.visualizeData3()
            self.stackedWidget.setCurrentIndex(2)
            

    def exportData(self):
        fileName = QFileDialog.getSaveFileName(caption='Save File', filter='*.xlsx')[0]
        if fileName != '':
            with pd.ExcelWriter(fileName, engine='xlsxwriter') as writer:
            
                for i in self.spectralDict:
                    pd.DataFrame(self.spectralDict[i]).to_excel(writer, 
                                                                sheet_name=str(i)
                                                                )
    
    def visualizeData(self):
        dictonary = dict()
        for i in self.spectralDict:
            dictonary[i] = {band: self.spectralDict[i][band] for band in self.bands}
        band = self.bandComboBox.currentText()
        if len(dictonary) != 0:
            if not self.plotShowed:
                self.figure = plt.figure()
                self.canvas = FigureCanvas(self.figure)
                self.toolbar = NavigationToolbar(self.canvas, self.boxPlotFrame)
                self.layout = QtWidgets.QVBoxLayout()
                self.layout.addWidget(self.toolbar)
                self.layout.addWidget(self.canvas)
                self.boxPlotFrame.setLayout(self.layout)
                self.ax = self.figure.add_subplot(111)
            
            self.ax.clear()
            self.ax.boxplot([dictonary[i][band] for i in dictonary],
                            labels=[i for i in dictonary])
            
            self.canvas.draw()
            self.plotShowed = True
        else:
            self.showDialog('nodata')
            
    def visualizeData2(self):
        from math import pi
        dictonary = dict()
        for i in self.spectralDict:
            dictonary[i] = {band: self.spectralDict[i][band] for band in self.bands}
            
        radarDict = {'group': [],
                     'RED': [],
                     'GREEN': [],
                     'BLUE': [],
                     'NIR': [],
                     'SWIR1': [],
                     'SWIR2': []}
        
        if not self.plot2Showed:
            self.figure2 = plt.figure()
            self.canvas2 = FigureCanvas(self.figure2)
            self.toolbar2 = NavigationToolbar(self.canvas2, self.radarPlotFrame)
            self.layout2 = QtWidgets.QVBoxLayout()
            self.layout2.addWidget(self.toolbar2)
            self.layout2.addWidget(self.canvas2)
            self.radarPlotFrame.setLayout(self.layout2)
            self.ax2 = self.figure2.add_subplot(111, polar=True)   
        
        self.ax2.clear()        
        
        for year in dictonary:
            radarDict['group'].append(year)
            for band in dictonary[year]:
                valueList = dictonary[year][band]
                valueListMean = sum(valueList)/len(valueList)
                radarDict[band].append(valueListMean)
                
        df = pd.DataFrame(radarDict)

        categories=list(df)[1:]
        N = len(categories)

        angles = [n / float(N) * 2 * pi for n in range(N)]
        angles += angles[:1]

        self.ax2.set_theta_offset(pi / 2)
        self.ax2.set_theta_direction(-1)

        self.ax2.set_xticks(angles[:-1], categories)
         
        for i in range(len(df.group.to_list())):
            values=df.loc[i].drop('group').values.flatten().tolist()
            values += values[:1]
            self.ax2.plot(angles, values, linewidth=1, linestyle='solid', label=f"{df.group.to_list()[i]}")
            self.ax2.fill(angles, values, alpha=0.1)

        self.ax2.legend(bbox_to_anchor=(1.5, 0.9))

        self.canvas2.draw()
        self.plot2Showed = True

    def visualizeData3(self):
          dictonary = dict()
          for i in self.spectralDict:
              dictonary[i] = {band: self.spectralDict[i][band] for band in self.bands}
          if len(dictonary) != 0:
              if not self.plot3Showed:
                  self.figure3, self.axs = plt.subplots(2, 3, sharex=True, sharey=True)
                  self.canvas3 = FigureCanvas(self.figure3)
                  self.toolbar3 = NavigationToolbar(self.canvas3, self.boxPlotFrame2)
                  self.layout3 = QtWidgets.QVBoxLayout()
                  self.layout3.addWidget(self.toolbar3)
                  self.layout3.addWidget(self.canvas3)
                  self.boxPlotFrame2.setLayout(self.layout3)
              
              c = 0
              x = 0
              y = 0
              for b in self.bands:
                  self.axs[x, y].clear()
                  self.axs[x, y].boxplot([dictonary[i][b] for i in dictonary],
                                         labels=[i for i in dictonary])
                  self.axs[x, y].set_title(b)
                  
                  if c < 2:
                      c+=1
                      y+=1
                  else:
                      y=0
                      x=1
                      c=0
                      
              self.figure3.autofmt_xdate(rotation=45)
              self.figure3.tight_layout()        
              self.canvas3.draw()
              self.plot3Showed = True
          else:
              self.showDialog('nodata')  
              
              
class AppWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = MainWindow()
        self.ui.setupUi(self)
        self.ui.init(self)
        self.ui.interactions()
        

if __name__ == '__main__':
    app = QApplication(sys.argv)
    app.processEvents()
    w = AppWindow()
    w.show()
    sys.exit(app.exec_())