# -*- coding: utf-8 -*-
"""
Created on Fri Mar 18 16:07:51 2022

@author: michael
"""

import ee, geopandas as gpd, pandas as pd
import ndop as nd
from io import StringIO
import csv

def getOccurrences(taxonName):
    def getNdopData(s,num_rec,table_payload):
        csv_table = []
        counter = 0

        for i in range(0, num_rec, 500):            
            counter+=1
            table_url = (
                'https://portal.nature.cz/nd/find.php?'
                'akce=seznam&opener=&vztazne_id=0&order=ID_ND_NALEZ'
                '&orderhow=DESC&frompage={frompage}&pagesize=500&'
                'filtering=&searching=&export=1&ndtoken={ndtoken}'
            ).format(frompage=str(i), ndtoken=table_payload['ndtokenexport'])

            req = s.post(url=table_url, data=table_payload)
            req.encoding = 'cp1250'
            f = StringIO(req.text)
            reader = csv.reader(f, delimiter=';')
            if counter == 1: 
                csv_table.append(list(reader))
            else:
                csv_table.append(list(reader)[1:])

        ndopData = sum(csv_table,[])
        return ndopData

    search_payload = nd.get_search_pars()
    search_payload.update({'rfTaxon':f'{taxonName}'})

    username = "michael.kuceraaa@gmail.com"
    password = "Ei1qM1aPi"


    s = nd.login(username, password)
    table_payload, num_rec = nd.search_filter(s,search_payload)

    ndopData = getNdopData(s,
                    num_rec,
                    table_payload
                    )

    ndopDf = pd.DataFrame(data=ndopData[1:], columns=ndopData[0])

    souradniceX = ndopDf.X.to_list()
    souradniceY = ndopDf.Y.to_list()
    presnost = ndopDf.CXPRESNOST.to_list()
    datum = ndopDf.DATUM_OD.to_list()
    
    dfDict = {'X':[],
              'Y':[],
              'presnost':[],
              'datum':[]
              }
    
    for i in range(len(souradniceY)):
        try:
            dfDict['X'].append(float(souradniceX[i].replace(',', '.')))
            dfDict['Y'].append(float(souradniceY[i].replace(',', '.')))
            dfDict['presnost'].append(int(presnost[i]))
            dfDict['datum'].append(int(datum[i]))
        except:
            None
    
    df = pd.DataFrame(columns=['X', 'Y', 'CXPRESNOST', 'DATUM_OD'])
    df.X = dfDict['X']
    df.Y = dfDict['Y']
    df.CXPRESNOST = dfDict['presnost']
    df.DATUM_OD = dfDict['datum']
        
    gdf = gpd.GeoDataFrame(df, geometry=gpd.points_from_xy(df.X, df.Y)).set_crs('epsg:5514').to_crs('epsg:4326')
    
    return gdf
    
def getSpectralValues(yearParams, filterParams):
    spectralDict = dict()
    try:
        ee.Initialize()
    except:
        ee.Authenticate()
        ee.Initialize()
    
    def maskClouds(image):    
        qa = image.select('pixel_qa')  
        maskedImage = image.updateMask(qa.bitwiseAnd(1 << 3).eq(0)).updateMask(qa.bitwiseAnd(1 << 5).eq(0))    
        return maskedImage   
    
    def getLandsat():
        l8 = ee.ImageCollection('LANDSAT/LC08/C01/T1_SR') \
                .filter(ee.Filter.calendarRange(5, 8, 'month')) \
                .map(maskClouds) \
                .select(['B2', 'B3', 'B4', 'B5', 'B6', 'B7'],
                        ['BLUE', 'GREEN', 'RED', 'NIR', 'SWIR1', 'SWIR2']
                        )
        
        l7 = ee.ImageCollection('LANDSAT/LE07/C01/T1_SR') \
                .filter(ee.Filter.calendarRange(5, 8, 'month')) \
                .map(maskClouds) \
                .select(['B1', 'B2', 'B3', 'B4', 'B5', 'B7'],
                        ['BLUE', 'GREEN', 'RED', 'NIR', 'SWIR1', 'SWIR2']
                        )
        
        l5 = ee.ImageCollection('LANDSAT/LT05/C01/T1_SR') \
                .filter(ee.Filter.calendarRange(5, 8, 'month')) \
                .map(maskClouds) \
                .select(['B1', 'B2', 'B3', 'B4', 'B5', 'B7'],
                        ['BLUE', 'GREEN', 'RED', 'NIR', 'SWIR1', 'SWIR2']
                        )
        
        l4 = ee.ImageCollection('LANDSAT/LT04/C01/T1_SR') \
                .filter(ee.Filter.calendarRange(5, 8, 'month')) \
                .map(maskClouds) \
                .select(['B1', 'B2', 'B3', 'B4', 'B5', 'B7'],
                        ['BLUE', 'GREEN', 'RED', 'NIR', 'SWIR1', 'SWIR2']
                        )
                
        landsatCollection = l4.merge(l5).merge(l7).merge(l8)
        
        return landsatCollection
    
    def filterOccurrences(): 
        filterType = [filterParams[0],
                      filterParams[0][filterParams[0]['CXPRESNOST'] == filterParams[1]],
                      filterParams[0][filterParams[0]['CXPRESNOST'] < filterParams[1]],
                      filterParams[0][filterParams[0]['CXPRESNOST'] > filterParams[1]],
                      filterParams[0][filterParams[0]['CXPRESNOST'].isin(list(range(filterParams[1], filterParams[2]+1)))],
                      filterParams[0][filterParams[0]['CXPRESNOST'] != filterParams[1]],
                      filterParams[0][~filterParams[0]['CXPRESNOST'].isin(list(range(filterParams[1], filterParams[2]+1)))]
                      ]
        
        return filterType[filterParams[3]]
    
    landsatCollection = getLandsat()
    gdf = filterOccurrences()
    output = list()
    yearList = list()
    years = list(range(yearParams[0],
                       yearParams[1]+1,
                       yearParams[2]
                       )
                 )

    yearsF = [int(str(j)+('0101')) for j in years]
    
    for yearInx in range(len(years)-1):
        gdfYear = gdf[gdf.DATUM_OD.isin(list(range(yearsF[yearInx],
                                                   yearsF[yearInx+1]
                                                   )
                                             )
                                        )
                      ]
        
        image = landsatCollection.filterDate(ee.Date.fromYMD(years[yearInx],1,1),
                                             ee.Date.fromYMD(years[yearInx+1],1,1)
                                             ).median()
        
        chunks = len(gdfYear) // 5000 + 1
        results = list()      
        
        for i in range(chunks):
            gdfYearChunk = gdfYear[i*5000:(i+1)*5000]
            eeYearChunk = ee.FeatureCollection([ee.Feature(feature) for feature in gdfYearChunk.iterfeatures()])
            
            try: 
                result = pd.read_csv(image.sampleRegions(collection=eeYearChunk,
                                                         scale=30,
                                                         tileScale=4
                                                         ).getDownloadURL())[['DATUM_OD',
                                                                              'CXPRESNOST',
                                                                              'X',
                                                                              'Y',
                                                                              'BLUE',
                                                                              'GREEN',
                                                                              'RED',
                                                                              'NIR',
                                                                              'SWIR1',
                                                                              'SWIR2'
                                                                              ]
                                                                              ]
         
                results.append(result)
            except:
                None
                
        if len(results) != 0:        
            df = pd.concat([i for i in results])
            output.append(df)
            yearList.append(years[yearInx])
        
    spectralDict = dict()
    for i in range(len(output)):
        spectralDict[yearList[i]] = dict()
        for band in ['BLUE', 'GREEN', 'RED', 'NIR', 'SWIR1', 'SWIR2', 'CXPRESNOST', 'DATUM_OD']:
            spectralDict[yearList[i]][band] =  output[i][band].to_list()
      
    for band in ['BLUE', 'GREEN', 'RED', 'NIR', 'SWIR1', 'SWIR2']:    
        normList = list()
        for i in spectralDict:     
            normList.extend(spectralDict[i][band])
        bandMin = min(normList)
        bandMax = max(normList)
        for i in spectralDict:
            spectralDict[i][band] = [((x - bandMin)/(bandMax - bandMin)) for x in spectralDict[i][band]]        
        
    return spectralDict